//temporalRowEntry.hxx
//
//Chris Bentzel 01.16.98

/***************************************************************************
 * 
 * Copyright 1998  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#ifndef _TEMPORAL_ROW_ENTRY_HDR_
#define _TEMPORAL_ROW_ENTRY_HDR_

#include <string.h>
//string.h needed for memcpy function

class temporalRowEntry {

public:

  //constructors
  temporalRowEntry();
  temporalRowEntry(const temporalRowEntry& tRE);
  temporalRowEntry(void* data, int dataSize);

  //destructor
  ~temporalRowEntry();

  //assignment
  temporalRowEntry& operator=(const temporalRowEntry& tRE);

  //setting data
  void set(void* data, int dataSize);

  //reading data
  void* getData();
  int getDataSize() const;

  //for debugging only
  int getReferenceCount() const;
  
private:

  //temporalRowEntryRep is for internal use only
  class temporalRowEntryRep {
  public:
    void* d;
    int dS;
    int referenceCount;

    temporalRowEntryRep()
    {
      d = 0;
      dS = 0;
      referenceCount = 1;
    }
    
    temporalRowEntryRep(void* data, int dataSize)
    {
      //create data size
      d =(void *)new char[dataSize];
      memcpy(d, data, dataSize);
      dS = dataSize;
      referenceCount = 1;
    }
    
    ~temporalRowEntryRep()
    {
      delete[] d;
    }
    
  };
  
  temporalRowEntryRep* rep;
};





















//inlines

inline
temporalRowEntry::temporalRowEntry()
{
  rep = new temporalRowEntryRep();
}

inline
temporalRowEntry::temporalRowEntry(const temporalRowEntry& tRE)
{
  //point to the same data
  rep = tRE.rep;
  rep->referenceCount++;
}

inline
temporalRowEntry::temporalRowEntry(void* data, int dataSize)
{
  rep = new temporalRowEntryRep(data, dataSize);
}

inline
temporalRowEntry::~temporalRowEntry()
{
  if (--rep->referenceCount == 0)
    {
      delete rep;
    }
}

inline temporalRowEntry&
temporalRowEntry::operator=(const temporalRowEntry& tRE)
{
  //check to make sure the assignment is not a=a
  if (&tRE != this)
    {
      //delete the current rep if no one else referencing it
      if (--rep->referenceCount == 0)
        {
          delete rep;
        }
      //increase the assigned referenceCount
      tRE.rep->referenceCount++;
      //copy over address
      rep = tRE.rep;
    }

  return *this;
}

inline void*
temporalRowEntry::getData()
{
  //probably want to make this safer... don't expose the rep
  return rep->d;
}

inline int
temporalRowEntry::getDataSize() const
{
  return rep->dS;
}

inline int
temporalRowEntry::getReferenceCount() const
{
  return rep->referenceCount;
}

inline void
temporalRowEntry::set(void* data, int dataSize)
{
  //decrement reference count and clear memory if necessary
  if (--rep->referenceCount == 0)
    {
      delete rep;
    }

  //assign the new type
  rep = new temporalRowEntryRep(data, dataSize);
}

#endif // _TEMPORAL_ROW_ENTRY_REFERENCE_HDR_











