//visTclCommands.c++
//Chris Bentzel
//Sep. 97

/***************************************************************************
 * 
 * Copyright 1997  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

/////////////////////////////////////////////////////////////////
////
////   Pre-processor directives, includes, globals, datatypes
////
/////////////////////////////////////////////////////////////////
#include "CVisStateHandler.h++"
#include "comGUI.h++"
#include "pvm3.h"
#include "messageTags.h++"
#include <iostream.h>
#include <unistd.h>
#include <string.h>
#include <tk.h>

extern CVisState* theBackupState;
extern CVisState* theCurrentState;
extern int visStateLockedByGUI;
extern int currentNumberOfObjects;
extern int currentNumberOfPeople;
extern int currentNumberOfDetectorItems;

int whichElemLocked = -1;
int whichElemDetecting = -1;
int whichElemEvent = -1;
int whichElemType = -1;
int pvmIsFucked = 0;
int noMessageIn;

enum {GUI_CUBE_TYPE, GUI_PERSON_TYPE, GUI_DETECTOR_TYPE};



/////////////////////////////////////////////////////////////////
////
////   getElemsCmd()
////
/////////////////////////////////////////////////////////////////
int
getElemsCmd(ClientData , Tcl_Interp* interp,
            int argc, char** )
{
  if (argc != 1)
    {
      interp->result = "Wrong # of args!";
      return TCL_ERROR;
    }

  strcpy(interp->result, "");
  char tmpString[32];
  int totalDetectorItems = theCurrentState->getNumberOfDetectorItems();
  for (int i = 0; i < totalDetectorItems; i++)
    {
      theCurrentState->visIntToString(i, tmpString); 
      strcat(interp->result, tmpString);
      if (i < totalDetectorItems - 1)
        strcat(interp->result, ".");
    }
  return TCL_OK;
}



/////////////////////////////////////////////////////////////////
////
////   getNumberOfObjects
////
/////////////////////////////////////////////////////////////////
int
getNumberOfObjectsCmd(ClientData , Tcl_Interp* interp,
                   int argc, char** )
{
  if (argc != 1)
    {
      Tcl_SetResult(interp, "No arguments", TCL_STATIC);
      return TCL_ERROR;
    }

  //return the number from the most recent struct
  sprintf(interp->result, "%d", theCurrentState->getNumberOfCubes());
  return TCL_OK;
}



/////////////////////////////////////////////////////////////////
////
////   readStateCmd()
////
/////////////////////////////////////////////////////////////////
int
readStateCmd(ClientData, Tcl_Interp* interp,
             int , char**)
{
  //get the visState ID number
  int visStateID = pvm_gettid("visState", 0);

  //If there is a problem in obtaining the ID number,
  //see if the problem is simply that the group does not
  //exist. Otherwise, PVM is in trouble
  if (visStateID <= 0)
    {
      if (visStateID == PvmNoGroup)
        {
          interp->result = "visState not registered with PVM";
          return TCL_ERROR;
        }
      else
        {
          interp->result = "PVM is fucked!";
          return TCL_ERROR;
        }
    }

  //clear the send buffer and
  //send a read request to visState
  pvm_initsend(PvmDataDefault);
  pvm_send(visStateID, VISSTATE_READ_REQUEST_MSGTAG);

  //everything's OK
  return TCL_OK;
}



/////////////////////////////////////////////////////////////////
////
////   readAndLockStateCmd()
////
/////////////////////////////////////////////////////////////////
int
readAndLockStateCmd(ClientData , Tcl_Interp* interp,
                    int , char** )
{

  //Don't need to go anye further if we already locked the state
  if (visStateLockedByGUI)
    {
      interp->result = "The GUI has already locked the state!";
      return TCL_ERROR;
    }
  
  //Get the ID number for the visState process
  int visStateID = pvm_gettid("visState", 0);

  //If the ID number does not exist...
  if (visStateID <= 0)
    {
      //...because it has not registered with PVM yet, have
      //a minor problem
      if (visStateID == PvmNoGroup)
        {
          interp->result = "visState not registered with PVM!";
          return TCL_ERROR;
        }
      //...because there is a PVM error, there is a big problem
      else
        {
          interp->result = "PVM is fucked!";
          return TCL_ERROR;
        }
    }

  //Clear the send buffer and send out a readAndLock request
  pvm_initsend(PvmDataDefault);
  pvm_send(visStateID, VISSTATE_READ_AND_LOCK_REQUEST_MSGTAG);

  //We're done... let checkMessageQueue take care of the rest
  return TCL_OK;
}



/////////////////////////////////////////////////////////////////
////
////   modifyStateCmd
////
/////////////////////////////////////////////////////////////////
int
modifyStateCmd(ClientData , Tcl_Interp* interp,
            int , char** )
{
  
  //need to lock state first
  if (!visStateLockedByGUI)
    {
      interp->result = "Shit is going down.";
      
      //should be an error, but it seems to fuck up
      return TCL_OK;
    }
  else
    {
      //see if visState exists
      int visStateID = pvm_gettid("visState", 0);

      if (visStateID <= 0)
        {
          if (visStateID == PvmNoGroup)
            {
              interp->result = "visState not registered with PVM!";
              return TCL_ERROR;
            }
          else
            {
              interp->result = "PVM is fucked!";
              return TCL_ERROR;
            }
        }
      else
        {
          //get the state from TCL
          copyTclToState((void *)interp);

          //clear the send buffer, pack the state, and send it off
          pvm_initsend(PvmDataDefault);
          theCurrentState->pvmPack();
          pvm_send(visStateID, VISSTATE_MODIFY_REQUEST_MSGTAG);

          //we're golden
          interp->result = "";
          return TCL_OK;
        }
    }
}




/////////////////////////////////////////////////////////////////
////
////   panicCmd
////
/////////////////////////////////////////////////////////////////
int
panicCmd(ClientData , Tcl_Interp* interp, int , char** )
{
  int visStateID = pvm_gettid("visState", 0);
  
  if (visStateID <= 0)
    {
      if (visStateID == PvmNoGroup)
        {
          interp->result = "visState not registered with PVM!";
          return TCL_ERROR;
        }
      else
        {
          interp->result = "PVM is fucked!";
          return TCL_ERROR;
        }
    }
  
  pvm_initsend(PvmDataDefault);
  pvm_send(visStateID, VISSTATE_PANIC_REQUEST_MSGTAG);

  interp->result = "";
  return TCL_OK;
}




