// DriveTrainDoc.cpp : implementation of the CDriveTrainDoc class
// Chris Bentzel

/***************************************************************************
 * 
 * Copyright 1996  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#include "stdafx.h"
#include "DriveTrain.h"

#include "DriveTrainDoc.h"
#include "DriveTrainView.h"
#include "DriveTrainDialog.h"
#include "DriveTrainTrainDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDriveTrainDoc

IMPLEMENT_DYNCREATE(CDriveTrainDoc, CDocument)

BEGIN_MESSAGE_MAP(CDriveTrainDoc, CDocument)
	//{{AFX_MSG_MAP(CDriveTrainDoc)
	ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
	ON_UPDATE_COMMAND_UI(ID_FILE_IMPORT, OnUpdateFileImport)
	ON_UPDATE_COMMAND_UI(IDM_VIEWPOINTS, OnUpdateViewpoints)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_FILE_RELOAD, OnFileReload)
	ON_UPDATE_COMMAND_UI(ID_FILE_RELOAD, OnUpdateFileReload)
	ON_COMMAND(ID_TRAIN_MODE, OnTrainMode)
	ON_COMMAND(ID_DONE_POSING, OnDonePosing)
	ON_UPDATE_COMMAND_UI(ID_DONE_POSING, OnUpdateDonePosing)
	ON_UPDATE_COMMAND_UI(ID_TRAIN_MODE, OnUpdateTrainMode)
	ON_COMMAND(ID_MAP_OPEN, OnMapOpen)
	ON_COMMAND(ID_COLLECT_DATA, OnCollectData)
	ON_UPDATE_COMMAND_UI(ID_COLLECT_DATA, OnUpdateCollectData)
	ON_COMMAND(ID_MAP_CLEAR, OnMapClear)
	ON_COMMAND(ID_COLLECT_SEQUENCE, OnCollectSequence)
	ON_UPDATE_COMMAND_UI(ID_COLLECT_SEQUENCE, OnUpdateCollectSequence)
	ON_COMMAND(ID_MAP_NEW, OnMapNew)
	ON_UPDATE_COMMAND_UI(ID_MAP_NEW, OnUpdateMapNew)
	//ON_COMMAND(ID_YURI_OPEN, OnYuriOpen)
	//ON_COMMAND(ID_PLAY_YURI, OnPlayYuri)
	ON_COMMAND(ID_MAP_SAVEAS, OnMapSaveAs)
	ON_COMMAND(ID_MAPS_SAVECURRENTMAP, OnMapsSaveCurrentMap)
	//ON_UPDATE_COMMAND_UI(ID_PLAY_YURI, OnUpdatePlayYuri)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDriveTrainDoc construction/destruction

CDriveTrainDoc::CDriveTrainDoc()
{
  
  //standard data for all
  m_bContainsCamera = FALSE;
  m_eOpenType = IVFDOC_OPEN_NORMAL;
  CurrentState = CDriveTrainDoc::DRIVE_MODE;
  InTrakInstalled = theApp.getInTrakInstalled();
  NumberOfTicks = 0;
  MaxNumberOfTicks = 120;
  SequenceTicks = 120;
  SequenceWaitTicks = 180;
  
  //need to define several values if we have an InTrak
  if (InTrakInstalled)
    {
      //InsideTrakTransform::initClass();
      
      InTrakXform1 = new InsideTrakTransform();
      InTrakXform1->station.setValue(1);
      InTrakXformSensor = new SoNodeSensor();
      InTrakXformSensor->setFunction(InTrakSensorCB);
      InTrakXformSensor->setData(this);
      InTrakXformSensor->attach(InTrakXform1);
      
      // only need one sensor for both, since both are updated in 
      // one step by a static member function of InsideTrakTransform.
      
      InTrakXform2 = new InsideTrakTransform();
      InTrakXform2->station.setValue(2);
      InTrakXform2->ref();
    }
  
  // create modeless dialog box
  if (InTrakInstalled)
	  {
	  dialog = new CDriveTrainDialog(this);
	  dialog->Create();
	  }
  
  input = zeros(1,15);
  lastInput = zeros(1,15);  
  
  //m_model = new Model();
  //yuri_loaded = FALSE;
  //m_SplineDialog = new CSplineDialog(this);
  //m_SplineDialog->Create();
  //m_SplineDialog->UpdateData(TRUE);	
  //YuriSeqMade = FALSE;
  //	M<double> tmp_map(0, 1, 24);
  //yuriInitValues = tmp_map;
}

CDriveTrainDoc::~CDriveTrainDoc()
	{
	if (InTrakInstalled)
		{
		InTrakXformSensor->detach();
	
		int res;
		dialog->EndDialog(res);
		}
	//int fe;
	//m_SplineDialog->EndDialog(fe);
	//delete(m_model);
	//if (YuriSeqMade)
	//	{
	//	for (int i = 0; i < YuriSeqM; i++)
	//		delete[] YuriSequence[i];
	//	delete[] YuriSequence;
	//	}
	}
	

BOOL 
CDriveTrainDoc::OnNewDocument()
{
  if (!CDocument::OnNewDocument())
    return FALSE;
  
  // BEGIN_IVWGEN
  IvfOnNewDocument();
  // END_IVWGEN
   
  // set up a Quadraped
  SoSeparator *root = new SoSeparator;
  root->ref();
  
  SoMaterial  *myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue( 1., 0.5, 0. );
  root->addChild( myMaterial );
  
  // set up a quadraped
  q = new Quadraped;
  
  // set up connections to quadraped
  qMap = new QuadrapedMap(q);
  //qMap->read(yuriInitValues);
  created_maps = 0;

  // add it to the scene graph
  root->addChild( q );
  IvfSetSceneGraph( root );
  root->unref();


  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDriveTrainDoc serialization

void 
CDriveTrainDoc::Serialize(CArchive& ar)
	{
	if (ar.IsStoring())
		{
		// BEGIN_IVWGEN
		//DriveTrainOnSaveDocument(LPCTSTR(ar.GetFile()->GetFilePath()));
        IvfOnSaveDocument(ar.GetFile()->m_hFile) ;
		// END_IVWGEN
 
		}
	else
		{
		// BEGIN_IVWGEN
		//DriveTrainOnOpenDocument(LPCTSTR(ar.GetFile()->GetFilePath()));
		IvfOnOpenDocument(ar.GetFile()->m_hFile) ;
		// END_IVWGEN
		}
	}

/////////////////////////////////////////////////////////////////////////////
// CDriveTrainDoc diagnostics

#ifdef _DEBUG
void CDriveTrainDoc::AssertValid() const
	{
	CDocument::AssertValid();
	}

void CDriveTrainDoc::Dump(CDumpContext& dc) const
	{
	CDocument::Dump(dc);
	}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDriveTrainDoc commands




//  OnFileImport --
//  yet another special file open scenario.  still want to route
//  through OpenDocument to handle menus, urls, etc., but don't
//  want to actually update the document per se.  This just gets
//  added to the current doc.
//

void 
CDriveTrainDoc::OnFileImport()
	{
	m_eOpenType = IVFDOC_OPEN_IMPORT;
	theApp.OnFileOpen();
	}


void 
CDriveTrainDoc::OnUpdateFileImport(CCmdUI *pCmdUI)
	{
	pCmdUI->Enable(m_pSceneRoot != NULL);
	}


BOOL 
CDriveTrainDoc::OnOpenDocument(LPCTSTR lpszPathName) 
	{
	if (!IvfOnOpenDocumentMessage(lpszPathName))
		return FALSE;

	SoSeparator *root = IvfGetDocSceneGraph();

	// find a quadraped
	Quadraped * q;
	SoSearchAction *s = new SoSearchAction;
	s->setType(Quadraped::getClassTypeId(), TRUE);
	s->apply(root);
	SoPath * path = s->getPath();

	if (!path)
	   	// error box
		return FALSE;
	else
		q = (Quadraped *) path->getTail();	

	// set up connections to quadraped
	qMap = new QuadrapedMap(q);
	//qMap->read(yuriInitValues);
	created_maps = 0;
	return TRUE;

	}


void 
CDriveTrainDoc::OnUpdateViewpoints(CCmdUI* pCmdUI) 
	{
	IvfOnUpdateViewpoints(pCmdUI);
	}


//static char spv_save_suffix[] = ".dt";
static char spv_save_suffix[] = ".iv";
//static char spv_save_filter[] =
//   "DriveTrain Files (*.dt)\0*.dt\0Inventor Files (*.iv)\0*.iv\0Vrml Files (*.wrl)\0*.wrl\0BMP Files (*.bmp)\0*.bmp\0";
static char spv_save_filter[] =
   "Inventor Files (*.iv)\0*.iv\0Vrml Files (*.wrl)\0*.wrl\0BMP Files (*.bmp)\0*.bmp\0";


void 
CDriveTrainDoc::OnFileSaveAs() 
	{
	CFileDialog save_dlg(FALSE);
    //  create the dialog

	save_dlg.m_ofn.lpstrFilter = spv_save_filter ;
	save_dlg.m_ofn.nFilterIndex = 1L;
	save_dlg.m_ofn.lpstrCustomFilter = NULL;
	save_dlg.m_ofn.lpstrDefExt = spv_save_suffix+1;
	save_dlg.m_ofn.nMaxCustFilter = 0;
        save_dlg.m_ofn.lpstrFile[0] = '\0';

	save_dlg.m_ofn.Flags |= (OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT) ;


	if (save_dlg.DoModal() != IDCANCEL)
		{
		BOOL stat = FALSE;
		// Note: Saving may change the file type, e.g. from Inventor to VRML 
		switch (save_dlg.m_ofn.nFilterIndex)
			{
			//case 1:   // .dt file
			//	//stat = DriveTrainOnSaveDocument(save_dlg.m_ofn.lpstrFile);
			//	break;

			case 1: 	// .iv file
				IvfSetFileType(IVF_FILETYPE_IV);
				stat = OnSaveDocument(save_dlg.m_ofn.lpstrFile);
				break;
			// Set filetype to Inventor in case it was VRML.

			case 2:	// .wrl file
				if (IvfGetFileType() == IVF_FILETYPE_IV)
				IvfSetFileType(IVF_FILETYPE_VRML);
				stat = OnSaveDocument(save_dlg.m_ofn.lpstrFile);
				break;
				//  VRML requires new header - need to set the filetype.
                //  Don't change if file was read in as VRML or VRML2.

			case 3:	// .bmp file
				{
				CDriveTrainView *t_view =(CDriveTrainView *)CIvfApp::IvfGetAfxView();
				stat = t_view->SaveAsBitmap(save_dlg.m_ofn.lpstrFile);
				}
			break;
				//  .bmp is totally different - let view handle.
			}
		}
	}


BOOL 
CDriveTrainDoc::OnSaveDocument(LPCTSTR lpszPathName) 
	{
	BOOL stat;
	CDriveTrainView *pView =(CDriveTrainView *)CIvfApp::IvfGetAfxView();
	pView->IvfRemoveManips();
	stat = CDocument::OnSaveDocument(lpszPathName);
	pView->IvfRestoreManips();
	return(stat);
	}


void 
CDriveTrainDoc::OnFileReload() 
	{
	CIvfApp *pApp = CIvfApp::IvfGetApp();
	IvfDeleteContents();
	OnOpenDocument(GetPathName());
	if (pApp->IvfIsMdi()) IvfSceneGraphChanged();
			//  an mdi app needs to treat the single
			//  frame as an SDI to reload.
	}


void 
CDriveTrainDoc::OnUpdateFileReload(CCmdUI* pCmdUI) 
	{ pCmdUI->Enable(m_pSceneRoot != NULL);}

inline void 
CDriveTrainDoc::IvfSetupUrlFetchCallback(void)
	{ CIvfDocument::IvfSetupUrlFetchCallback();}


inline void 
CDriveTrainDoc::OnTrainMode() 
	{ setCurrentState(CDriveTrainDoc::SNAPSHOT_MODE); }

inline int
CDriveTrainDoc::getCurrentState()
	{ return CurrentState; }

inline void
CDriveTrainDoc::setCurrentState(int state)
	{ CurrentState = state; }

inline int 
CDriveTrainDoc::getNumberOfTicks()
	{ return NumberOfTicks; }

inline void
CDriveTrainDoc::setNumberOfTicks(int ticks)
	{ NumberOfTicks = ticks; }

inline int
CDriveTrainDoc::getMaxNumberOfTicks()
	{ return MaxNumberOfTicks; }

inline void
CDriveTrainDoc::setMaxNumberOfTicks(int ticks)
	{ MaxNumberOfTicks = ticks; }

void
CDriveTrainDoc::setInTrakData(M<double>& a)
	{
	SbVec3f t;
	SbRotation r;
	float x, y, z;
	M<double> tmp_mat;
  
	a = zeros(1,15);
  
	t = InTrakXform1->translation.getValue();
	t.getValue(x,y,z);
 
	// if these indices change, must change setPdfInput
	a(1,1) = x; a(1,2) = y; a(1,3) = z;
  
	t = InTrakXform2->translation.getValue();
	t.getValue(x,y,z);
	a(1,4) = x; a(1,5) = y; a(1,6) = z;
  
	if (InTrakInstalled)
		{
		a(1,7) = dialog->m_slider1.GetPos();
		a(1,8) = dialog->m_slider2.GetPos();
		a(1,9) = dialog->m_slider3.GetPos();
		}
	else
		{
		a(1, 7) = 0.0;
		a(1, 8) = 0.0;
		a(1, 9) = 0.0;
		} 
  
	r = InTrakXform1->rotation.getValue();
	qMap->Mat2Euler(r, tmp_mat);	
	a(1, colon(10, 1, 12)) = tmp_mat(1, colon(1, 1, 3));	

	r = InTrakXform2->rotation.getValue();
	qMap->Mat2Euler(r, tmp_mat);
	a(1, colon(13, 1, 15)) = tmp_mat(1, colon(1, 1, 3));
  
}

void
CDriveTrainDoc::setPdfInput(M<double>& x)
{
	M<double> idx = colon(1,1,6);
	x = zeros(1,12);
	x(1, idx) = input(1, idx);
	x(1, colon(7,1,12)) = input(1,idx) - lastInput(1, idx);
}


void
CDriveTrainDoc::tick()
{
  lastInput = input;
  setInTrakData(input);
  setPdfInput(pdfInput);
  
  switch (getCurrentState()) 
	  {
	  case (CDriveTrainDoc::DRIVE_MODE) :
		  {
		  if (created_maps > 0)
			  {
			  updateMapNumber();
			  if (listOfmaps[currentMapNumber]->numExamples() > 1)
				  qMap->write(listOfmaps[currentMapNumber]->useMap(input));
			  }
		  break;
		  }
	  case (CDriveTrainDoc::SNAPSHOT_MODE) :
		  {
		  double speed = norm(input - lastInput).r(1,1);
		  if ( (getNumberOfTicks() == getMaxNumberOfTicks() )
			  || ((speed < 0.10) && (getNumberOfTicks() > 15)) )
			  {
			  setNumberOfTicks(0);
			  setInTrakData(snapshot);
			  setCurrentState(CDriveTrainDoc::POSE_MODE);
			  if ((dialog->m_setInitial) && (listOfmaps[currentMapNumber]->numExamples() > 1))
				  qMap->write(listOfmaps[currentMapNumber]->useMap(snapshot));
			  break;
			  }
		  else
			  setNumberOfTicks(getNumberOfTicks() + 1);				
		  break;
		  }
    case (CDriveTrainDoc::POSE_MODE) :
        break;
    case (CDriveTrainDoc::COLLECT_MODE) :
		{
        // probably ought to have a max on it.
        collectedData(nextCollectedData++, c_p) = pdfInput;
        break;
		}
	case (CDriveTrainDoc::SEQUENCE_WAIT_MODE) :
		{
		if (NumberOfTicks < SequenceWaitTicks)
			NumberOfTicks++;
		else
			{
			NumberOfTicks = 0;
			setCurrentState(CDriveTrainDoc::SEQUENCE_MODE);
			}
		break;
		}
	case (CDriveTrainDoc::SEQUENCE_MODE) :
		{
		if (NumberOfTicks < SequenceTicks)
			{
			NumberOfTicks++;
			M<double> tmp_fun(0, 1, 12);
			tmp_fun(1, c_p) = inputSeq(NumberOfTicks, c_p);
			setInTrakData(tmp_fun);
			}
		else
			{
			NumberOfTicks = 0;
			M<double> tmp_joke(0, SequenceTicks, 24);
			listOfmaps[currentMapNumber]->addData(inputSeq, tmp_joke);
			setCurrentState(CDriveTrainDoc::DRIVE_MODE);
			}	
		break;
		}
	//case (CDriveTrainDoc::YURI_MODE) :
	//	{
	//	if (NumberOfTicks < YuriSeqM)
	//		{
	//		M<double> tmp_fun(0, 1, 24);
	//		tmp_fun(1, colon(7, 1, 12)) = yuriInitValues(1, colon(7, 1, 12));
	//		tmp_fun.r(1, 3) = YuriSequence[NumberOfTicks][1]/5;
	//		tmp_fun.r(1, 2) = -YuriSequence[NumberOfTicks][2]/5 - 1;
	//		tmp_fun.r(1, 1) = -YuriSequence[NumberOfTicks][3]/5 + (.5*yuriInitValues.r(1, 2));
	//		tmp_fun.r(1, 6) = YuriSequence[NumberOfTicks][4]/5;
	//		tmp_fun.r(1, 5) = -YuriSequence[NumberOfTicks][5]/5 - 1;
	//		tmp_fun.r(1, 4) = -YuriSequence[NumberOfTicks][6]/5 + (.5*yuriInitValues.r(1, 2));
	//		tmp_fun(1, colon(13, 1, 24)) = yuriInitValues(1, colon(13, 1, 24));
	//		qMap->write(tmp_fun);
	//		NumberOfTicks++;
	//		}
	//	else
	//		{
	//		NumberOfTicks = 0;
	//		setCurrentState(CDriveTrainDoc::DRIVE_MODE);
	//		}
	//	break;
	//	}
	  }
    }


void 
CDriveTrainDoc::InTrakSensorCB(void* data, SoSensor* )
{
	CDriveTrainDoc* attached_doc = (CDriveTrainDoc *)data;
	attached_doc->tick();
}
	

void
CDriveTrainDoc::updateMapNumber()
{

  // change the current map only if enabled bit is on
  if (dialog->m_autoSelectMap)
    {
      // classify the input
      double max = 0.0;
      int maxi = 0;
      for (int i = 0; i < NUM_MAPS; i++)
        {
          if (((i == 0) && (dialog->m_enable1)) ||		// this is stupid, but how to 
              ((i == 1) && (dialog->m_enable2)) ||		// put dialog members into arrays???
              ((i == 2) && (dialog->m_enable3)) ||	
              ((i == 3) && (dialog->m_enable4)))	
            pmap[i] = listOfmaps[i]->mapPDF.get( pdfInput );
          else pmap[i] = 0.0;
          
          if (pmap[i] > max ) { max = pmap[i]; maxi = i; }
          
          //TRACE("map %d = %f \n", i, pmap[i]);
          // should check Enabled flag for each map in dlog
        }
      // do I need to call update here?
      
      if (maxi != dialog->m_selectedMapNumber) 
        {
          currentMapNumber = maxi;
          dialog->m_selectedMapNumber = currentMapNumber;
          dialog->UpdateData(FALSE);
        }
    }
  else

    currentMapNumber = dialog->m_selectedMapNumber;    
}


void CDriveTrainDoc::OnDonePosing() 
{

  M<double> pose("pose",1, 45);
  
  // get the pose 
  qMap->read(pose);
  
  //add the input and output to the appropriate map
  listOfmaps[currentMapNumber]->addData(snapshot, pose);
  if (listOfmaps[currentMapNumber]->numExamples() > 1)
	  listOfmaps[currentMapNumber]->teachMap();

  // return to drive mode
  CurrentState = CDriveTrainDoc::DRIVE_MODE;		
}

void CDriveTrainDoc::OnUpdateDonePosing(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable((getCurrentState() == CDriveTrainDoc::POSE_MODE) && (InTrakInstalled));	
}

void CDriveTrainDoc::OnUpdateTrainMode(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(( getCurrentState() == CDriveTrainDoc::DRIVE_MODE) && (InTrakInstalled) && (created_maps > 0));	
}

void CDriveTrainDoc::OnMapSaveAs() 
{
  static char spv_save_suffix[] = ".map";
  static char spv_save_filter[] =
    "Drive Train Map Files (*.map)\0*.map\0";
  
  
  CFileDialog save_dlg(FALSE);
  //  create the dialog
  
  save_dlg.m_ofn.lpstrFilter = spv_save_filter ;
  save_dlg.m_ofn.nFilterIndex = 1L;
  save_dlg.m_ofn.lpstrCustomFilter = NULL;
  save_dlg.m_ofn.lpstrDefExt = spv_save_suffix+1;
  save_dlg.m_ofn.nMaxCustFilter = 0;
  save_dlg.m_ofn.lpstrFile[0] = '\0';
  
  save_dlg.m_ofn.Flags |= (OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT) ;
  
  
  if (save_dlg.DoModal() != IDCANCEL)
    {
      BOOL stat = FALSE;
      switch (save_dlg.m_ofn.nFilterIndex)
        {
        case 1: 	// .map file
          CString fname = save_dlg.m_ofn.lpstrFile;
          int s = fname.ReverseFind('\\') + 1;
          CString	name = fname.Mid( s, fname.GetLength() - s );
          switch (currentMapNumber)
            {	
			  case 0: dialog->m_fname1 = name; dialog->m_fsaved1 = TRUE; break;
			  case 1: dialog->m_fname2 = name; dialog->m_fsaved2 = TRUE; break;
			  case 2: dialog->m_fname3 = name; dialog->m_fsaved3 = TRUE; break;
			  case 3: dialog->m_fname4 = name; dialog->m_fsaved4 = TRUE; break;
            }
			
          OnMapsSaveCurrentMap();
          dialog->UpdateData(FALSE);
          break;
          // Set filetype to Inventor in case it was VRML.
          
          break;
          //  .bmp is totally different - let view handle.
        }
    }
}




void CDriveTrainDoc::OnMapsSaveCurrentMap() 
	{
	// get the filename of the current map, save it
	BOOL already_on_disk;
	switch ( currentMapNumber)
		{
		case 0:
			already_on_disk = dialog->m_fsaved1;
			break;
		case 1:
			already_on_disk = dialog->m_fsaved2;
			break;
		case 2:
			already_on_disk = dialog->m_fsaved3;
			break;
		case 3:
			already_on_disk = dialog->m_fsaved4;
			break;
		}
	if (already_on_disk)
		{
		CString fname	;
	
		switch( currentMapNumber )
			{	
			case 0:
				fname = dialog->m_fname1; 
				break;
			case 1:
				fname = dialog->m_fname2; 
				break;
			case 2:
				fname = dialog->m_fname3; 
				break;
			case 3:
				fname = dialog->m_fname4; 
				break;	
			}
	
		listOfmaps[currentMapNumber]->writeFile(fname);
		}
	else
		{
		static char spv_save_suffix[] = ".map";
		static char spv_save_filter[] =
			"Drive Train Map Files (*.map)\0*.map\0";
  
  
		CFileDialog save_dlg(FALSE);
		//  create the dialog
  
		save_dlg.m_ofn.lpstrFilter = spv_save_filter ;
		save_dlg.m_ofn.nFilterIndex = 1L;
		save_dlg.m_ofn.lpstrCustomFilter = NULL;
		save_dlg.m_ofn.lpstrDefExt = spv_save_suffix+1;
		save_dlg.m_ofn.nMaxCustFilter = 0;
		save_dlg.m_ofn.lpstrFile[0] = '\0';
  
		save_dlg.m_ofn.Flags |= (OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT) ;
  
  
		if (save_dlg.DoModal() != IDCANCEL)
			{
			BOOL stat = FALSE;
			switch (save_dlg.m_ofn.nFilterIndex)
				{
				case 1: 	// .map file
				CString fname = save_dlg.m_ofn.lpstrFile;
				int s = fname.ReverseFind('\\') + 1;
				CString	name = fname.Mid( s, fname.GetLength() - s );
				switch (currentMapNumber)
					{	
					case 0: dialog->m_fname1 = name; dialog->m_fsaved1 = TRUE; break;
					case 1:	dialog->m_fname2 = name; dialog->m_fsaved2 = TRUE; break;
					case 2: dialog->m_fname3 = name; dialog->m_fsaved3 = TRUE; break;
					case 3: dialog->m_fname4 = name; dialog->m_fsaved4 = TRUE; break;
					}
			
				OnMapsSaveCurrentMap();
				dialog->UpdateData(FALSE);
				break;
				// Set filetype to Inventor in case it was VRML.
          
				break;
				//  .bmp is totally different - let view handle.
				}
			}

		}

	}

void CDriveTrainDoc::OnMapOpen() 
{
  static char spv_open_suffix[] = ".map";
  static char spv_open_filter[] =
    "Drive Train Map Files (*.map)\0*.map\0";
  
  CFileDialog open_dlg(TRUE);
  //  create the dialog
  
  open_dlg.m_ofn.lpstrFilter = spv_open_filter ;
  open_dlg.m_ofn.nFilterIndex = 1L;
  open_dlg.m_ofn.lpstrCustomFilter = NULL;
  open_dlg.m_ofn.lpstrDefExt = spv_open_suffix+1;
  open_dlg.m_ofn.nMaxCustFilter = 0;
  open_dlg.m_ofn.lpstrFile[0] = '\0';
  
  //open_dlg.m_ofn.Flags |= (OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT) ;
  
  
if (open_dlg.DoModal() != IDCANCEL)
    {
      BOOL stat = FALSE;
      switch (open_dlg.m_ofn.nFilterIndex)
        {
        case 1: 	// .map file
          CString fname = open_dlg.m_ofn.lpstrFile;
          int s = fname.ReverseFind('\\') + 1;
          CString	name = fname.Mid( s, fname.GetLength() - s );
          switch (currentMapNumber)
              {
              case 0: dialog->m_fname1 = name; dialog->m_fsaved1 = TRUE; dialog->m_enable1 = TRUE; break;
              case 1: dialog->m_fname2 = name; dialog->m_fsaved2 = TRUE; dialog->m_enable2 = TRUE; break;
              case 2: dialog->m_fname3 = name; dialog->m_fsaved3 = TRUE; dialog->m_enable3 = TRUE; break;
              case 3: dialog->m_fname4 = name; dialog->m_fsaved4 = TRUE; dialog->m_enable4 = TRUE; break;
			  }
		  	
		  listOfmaps[currentMapNumber]->readFile(open_dlg.m_ofn.lpstrFile);
          dialog->UpdateData(FALSE);
          break;
          break;
        }
    }
}
 

void CDriveTrainDoc::OnCollectData() 
{
  if (CurrentState != CDriveTrainDoc::COLLECT_MODE)
    {
      // yes/no dialog box
      int ans = AfxMessageBox("Do you want to collect data for the current map?", MB_YESNO, 0);
      if (ans == IDNO)
        return;
      
      // set up space for samples
      
      collectedData = M<double>("collectedData", 30 * 60 * 5, 12);
      nextCollectedData = 1;
      CurrentState = CDriveTrainDoc::COLLECT_MODE;
    }
  else
    {                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
      // train the pdf
      // might want to ignore the values of the extra parameters sliders
      // (since they have nothing to do with your motion)
      CDriveTrainTrainDialog * pd = new CDriveTrainTrainDialog();
      pd->Create();
      pd->m_trainProgress.SetRange(0, 9);
      pd->m_trainProgress.SetPos(0);
      listOfmaps[currentMapNumber]->mapPDF.trainInit(collectedData(colon(1,1,nextCollectedData-1),c_p),6);
      for (int it = 0; it < 10; it++)
        {
          pd->m_trainProgress.SetPos(it);
          listOfmaps[currentMapNumber]->mapPDF.train1(collectedData(colon(1,1,nextCollectedData-1),c_p));
        }
      pd->EndDialog(0);
	  
      CurrentState = CDriveTrainDoc::DRIVE_MODE;
    }
  
}

void CDriveTrainDoc::OnUpdateCollectData(CCmdUI* pCmdUI) 
	{
	if (InTrakInstalled && (created_maps > 0))
		pCmdUI->SetCheck(getCurrentState() == CDriveTrainDoc::COLLECT_MODE);
	else
		pCmdUI->Enable(FALSE);
	}

void CDriveTrainDoc::OnMapClear() 
{
  int ans = AfxMessageBox("Are you sure you want to clear the current map?\npdf will remain unchanged.", MB_YESNO, 0);
  if (ans == IDNO)
    return;
  
  listOfmaps[currentMapNumber]->clear();
  
  //Inputs[currentMapNumber] = nop_M;
  //Poses[currentMapNumber] = nop_M;
  //numexamples[currentMapNumber] = 0;  
}

void CDriveTrainDoc::OnCollectSequence() 
	{
	CCollectSequenceDialog tmp_dialog;
	if (tmp_dialog.DoModal() == IDOK)
		{
		SequenceWaitTicks = tmp_dialog.m_wait_time;
		SequenceTicks = tmp_dialog.m_sequence_time;
		NumberOfTicks = 0;
		M<double> tmp_m(0, SequenceTicks, 12);
		inputSeq = tmp_m;
		CurrentState = CDriveTrainDoc::SEQUENCE_WAIT_MODE;
		}
	}

void CDriveTrainDoc::OnUpdateCollectSequence(CCmdUI* pCmdUI) 
	{
	pCmdUI->Enable(InTrakInstalled && (CurrentState == CDriveTrainDoc::DRIVE_MODE) && (created_maps > 0));
	}

void CDriveTrainDoc::OnMapNew() 
{
	if (InTrakInstalled)
		{
		CNewMapDialog tmp_dialog;
		if (tmp_dialog.DoModal() == IDOK)
			{
			if (tmp_dialog.m_MapType == 0)
				{
				if (created_maps < 4)
					{
					linRBFAbs* tmp_map = new linRBFAbs();
					listOfmaps[created_maps] = (GenMap *)tmp_map;
					switch (created_maps) {
						case (0) :
							dialog->m_fname1 = tmp_dialog.m_MapName;
							dialog->m_enable1 = TRUE;
							break;
						case (1) :
							dialog->m_fname2 = tmp_dialog.m_MapName;
							dialog->m_enable2 = TRUE;
							break;
						case (2) :
							dialog->m_fname3 = tmp_dialog.m_MapName;
							dialog->m_enable3 = TRUE;
							break;
						case (3) :
							dialog->m_fname4 = tmp_dialog.m_MapName;
							dialog->m_enable4 = TRUE;
							break;
						}	
					dialog->UpdateData(TRUE);
					int tmp_flags = 0;
					if (tmp_dialog.m_BluePhi)
						tmp_flags |= GenMap::BLUEPHI;
					if (tmp_dialog.m_BluePsi)
						tmp_flags |= GenMap::BLUEPSI;
					if (tmp_dialog.m_BlueTheta)
						tmp_flags |= GenMap::BLUETHETA;
					if (tmp_dialog.m_BlueX)
						tmp_flags |= GenMap::BLUEX;
					if (tmp_dialog.m_BlueY)
						tmp_flags |= GenMap::BLUEY;
					if (tmp_dialog.m_BlueZ)
						tmp_flags |= GenMap::BLUEZ;
					if (tmp_dialog.m_GreenPhi)
						tmp_flags |= GenMap::GREENPHI;
					if (tmp_dialog.m_GreenPsi)
						tmp_flags |= GenMap::GREENPSI;
					if (tmp_dialog.m_GreenTheta)
						tmp_flags |= GenMap::GREENTHETA;
					if (tmp_dialog.m_GreenX)
						tmp_flags |= GenMap::GREENX;
					if (tmp_dialog.m_GreenY)
						tmp_flags |= GenMap::GREENY;
					if (tmp_dialog.m_GreenZ)
						tmp_flags |= GenMap::GREENZ;
					listOfmaps[created_maps]->flags = tmp_flags;

					currentMapNumber = created_maps;
					created_maps++;
					}
				}				
			}
		} //end InTrakInstalled
	else
		AfxMessageBox("InTrak not installed; can't create map", MB_OK, 0);

	}

void CDriveTrainDoc::OnUpdateMapNew(CCmdUI* pCmdUI) 
	{
	pCmdUI->Enable(created_maps < 4);
	}

//void CDriveTrainDoc::OnYuriOpen() 
//	{
//	static char spv_open_suffix[] = ".yur";
//	static char spv_open_filter[] =
//		"Yuri Files (*.yur)\0*.yur\0";
  
//	CFileDialog open_dlg(TRUE);
	//  create the dialog
  
//	open_dlg.m_ofn.lpstrFilter = spv_open_filter ;
//	open_dlg.m_ofn.nFilterIndex = 1L;
//	open_dlg.m_ofn.lpstrCustomFilter = NULL;
//	open_dlg.m_ofn.lpstrDefExt = spv_open_suffix+1;
//	open_dlg.m_ofn.nMaxCustFilter = 0;
//	open_dlg.m_ofn.lpstrFile[0] = '\0';
  
  //open_dlg.m_ofn.Flags |= (OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT) ;
  
  
//	if (open_dlg.DoModal() != IDCANCEL)
//		{
//		CString fname = open_dlg.m_ofn.lpstrFile;
//		int s = fname.ReverseFind('\\') + 1;
//		CString	name = fname.Mid( s, fname.GetLength() - s );
//		m_model->LoadData(LPCTSTR(name));
//		yuri_loaded = TRUE;
//		}
//	}

//void CDriveTrainDoc::OnPlayYuri() 
//	{
//	slider_vals[0] = m_SplineDialog->m_Slider1.GetPos();
//	slider_vals[1] = m_SplineDialog->m_Slider2.GetPos();
//	slider_vals[2] = m_SplineDialog->m_Slider3.GetPos();
//	slider_vals[3] = m_SplineDialog->m_Slider4.GetPos();
//	slider_vals[4] = m_SplineDialog->m_Slider5.GetPos();
//	slider_vals[5] = m_SplineDialog->m_Slider6.GetPos();
//	slider_vals[6] = m_SplineDialog->m_Slider7.GetPos();
//	slider_vals[7] = m_SplineDialog->m_Slider8.GetPos();
//	slider_vals[8] = m_SplineDialog->m_Slider9.GetPos();
//	slider_vals[9] = m_SplineDialog->m_Slider10.GetPos();
//	if (YuriSeqMade)
//		{
//		for (int i = 0; i < YuriSeqM; i++)
//			delete[] YuriSequence[i];
//		delete[] YuriSequence;
//		}
//	qMap->read(yuriInitValues);
//	YuriSeqMade = TRUE;
//	YuriSeqM = m_SplineDialog->m_NumberTicks;
//	YuriSequence = new(pdouble[YuriSeqM]);
//	for (int j = 0; j < YuriSeqM; j++)
//		YuriSequence[j] = new(double[7]);		
//	m_model->MakeSequence(YuriSequence, YuriSeqM, slider_vals, 10);
//	NumberOfTicks = 0;
//	CurrentState = CDriveTrainDoc::YURI_MODE;
//	}
//
//void CDriveTrainDoc::OnUpdatePlayYuri(CCmdUI* pCmdUI) 
//	{
//	pCmdUI->Enable(yuri_loaded && (CurrentState == CDriveTrainDoc::DRIVE_MODE));	
//	}
//
