//GenMap.h
//abstract class for all network types
//Chris Bentzel

/***************************************************************************
 * 
 * Copyright 1997  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#ifndef _GEN_MAP_H
#define _GEN_MAP_H

#ifndef _M_TYPE_DOUBLE
#define _M_TYPE_DOUBLE
#define m_type double
#endif

#include <afx.h>

#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

#include <matlib.h>
#include "mixture.h"

class GenMap
{
public:

	enum
		{
		BLUEX = 0x001,
		BLUEY = 0x002,
		BLUEZ = 0x004,
		BLUETHETA = 0x008,
		BLUEPHI = 0x010,
		BLUEPSI = 0x020,
		GREENX = 0x040,
		GREENY = 0x080,
		GREENZ = 0x100,
		GREENTHETA = 0x200,
		GREENPHI = 0x400,
		GREENPSI = 0x800,
		};

  //useMap:
  //args: Input Data Vector
  //results: returns a vector describing output behavior
  virtual M<double> useMap(M<double> inData) = 0;
  
  //teachMap
  //results: perfroms appropriate learning algorithm on map based on 
  //         all data
  virtual void teachMap() = 0;
  
  //addData
  //args: void pointers to appropriate input and output data.
  //results: adds the data point specified by input_data and output_data to
  //         complete lists of data points.
  virtual void addData(M<double> inData, M<double> outData) = 0;

  //numExamples
  //args: none
  //results: returns the number of examples that has been fed to the current map
  virtual int numExamples() = 0;

  //clear
  //args: none
  //results: clears all data included in the map
  virtual void clear() = 0;
  
  //writes or reads from file if appropriate map
  virtual void readFile(CString MapFileName) = 0;
  virtual void writeFile(CString OutFileName) = 0;
  
  //force:
  //args: string containing potential runtime typename
  //results: if typeName = GenMap, returns this
  //         otherwise returns NULL.
  virtual void* force(CString typeName);

  //the PDF for all to use
  Mixture mapPDF;
	int flags;
  
};

#endif  // _GEN_MAP_H


	
