//LinRBF.cpp
//Chris Bentzel

/***************************************************************************
 * 
 * Copyright 1997  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#include "LinRBF.h"


void*
linRBF::force(CString typeName)
	{
	if (typeName == "linRBF")
		return this;
	else
		return (GenMap::force(typeName));
	}

linRBF::linRBF()
	{
	h = 0;
	}

void
linRBF::addData(M<double> inData, M<double>outData)
	{
	if (h <= 0) //this is the first addition
		{
		win = size(inData).r(1, 2);
		wout = size(inData).r(1, 2);
		}

	int old_size = h;
	h += size(inData).r(1, 1);

	M<double> tmp_in(0, h, win);
	tmp_in(colon(1, 1, old_size), c_p) = X(colon(1, 1, old_size), c_p);
	tmp_in(colon(old_size+1, 1, h), c_p) = inData(colon(1, 1, h-old_size), c_p);
	X = tmp_in;

	M<double> tmp_out(0, h, wout);
	tmp_out(colon(1, 1, old_size), c_p) = Y(colon(1, 1, old_size), c_p);
	tmp_out(colon(old_size+1, 1, h), c_p) = outData(colon(1, 1, h-old_size), c_p);
	Y = tmp_out;
	}

void
linRBF::clear()
	{
	X = nop_M;
	Y = nop_M;
	W = nop_M;
	h = 0;
	}

void 
linRBF::writeFile(CString OutFileName)
{
  save ((LPCTSTR) OutFileName,
        &(X), 
        &(Y),
        &(mapPDF.getPj()),
        &(mapPDF.getmu()),
        &(mapPDF.getK())
        );
}

void
linRBF::readFile(CString MapFileName)
{
  M<double> Pj, Mu, K;
  
  load( (LPCTSTR) MapFileName, 
        &(X), 
        &(Y),
        &Pj,
        &Mu,
        &K );
  
  h = size(X).r(1, 1);
  
  teachMap();
  
  mapPDF.set(Pj, Mu, K);
}

void
linRBF::teachMap()
{
  if (h <= 0)
    return;

  //assuming ordinary least squares
  M<double> H(0, h, h);
  for (int i = 1; i <= h; i++)
    for (int j = 1; j <= h; j++)
      H(i, j) = rbfunc(j, X(i, c_p));

  W = inv(H) * Y;

  Xmin = min(X);
  Xmax = max(X);
}

M<double>
linRBF::useMap(M<double> inData)
{
  M<double> H(0, 1, h);
  for (int i = 1; i <= win; i++)
	  if (inData.r(1, i) < Xmin.r(1, i))
		inData.r(1, i) = Xmin.r(1, i);
  for (i = 1; i <= win; i++)
	  if (inData.r(1, i) > Xmax.r(1, i))
		inData.r(1, i) = Xmax.r(1, i);

  for (i = 1; i <= h; i++)
    H(1, i) = rbfunc(i, inData);

  return(H*W);
}

  
  

