//QuadrapedMap.cpp
//Chris Bentzel

/***************************************************************************
 * 
 * Copyright 1996  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#include "QuadrapedMap.h"

QuadrapedMap::QuadrapedMap(Quadraped * q)
{
	// set up the connections to the quadraped

	// arm and leg control points
	leftArmCps = (SoCoordinate3 *) q->getPart("leftArm.hose.controlPoints",FALSE);
	rightArmCps = (SoCoordinate3 *) q->getPart("rightArm.hose.controlPoints",FALSE);
	leftLegCps = (SoCoordinate3 *) q->getPart("leftLeg.hose.controlPoints",FALSE);
	rightLegCps = (SoCoordinate3 *) q->getPart("rightLeg.hose.controlPoints",FALSE);

	// rotation of hands and feet
	leftHand = (SoTransform *) q->getPart("leftArm.effector.transform",FALSE);
	rightHand = (SoTransform *) q->getPart("rightArm.effector.transform",FALSE);
	leftFoot = (SoTransform *) q->getPart("leftLeg.effector.transform",FALSE);
	rightFoot = (SoTransform *) q->getPart("rightLeg.effector.transform",FALSE);

	// rotation of torso
	torso = (SoTransform *) q->getPart("torsoXform",FALSE);

	// location of pelvis
	pelvis = (SoTransform *) q->getPart("pelvis.transform",FALSE);

	// rotation of head
	head = (SoTransform *) q->getPart("head.transform",FALSE);	
	

}


void
QuadrapedMap::write( M<double> a)
{
	SbRotation tmp_rot;

	// left arm
	leftArmCps->point.set1Value(0, SbVec3f( a.r(1,1), a.r(1,2), a.r(1,3)) );
	leftArmCps->point.set1Value(1, SbVec3f( a.r(1,4), a.r(1,5), a.r(1,6)) );

	// right arm
	rightArmCps->point.set1Value(0, SbVec3f( a.r(1,7), a.r(1,8), a.r(1,9)) );
	rightArmCps->point.set1Value(1, SbVec3f( a.r(1,10), a.r(1,11), a.r(1,12)) );

	// left leg
	leftLegCps->point.set1Value(0, SbVec3f( a.r(1,13), a.r(1,14), a.r(1,15)) );
	leftLegCps->point.set1Value(1, SbVec3f( a.r(1,16), a.r(1,17), a.r(1,18)) );

	// right leg
	rightLegCps->point.set1Value(0, SbVec3f( a.r(1,19), a.r(1,20), a.r(1,21)) );
	rightLegCps->point.set1Value(1, SbVec3f( a.r(1,22), a.r(1,23), a.r(1,24)) );
	
	//left hand
	tmp_rot = Euler2Mat(SbVec3f( a.r(1, 25), a.r(1, 26), a.r(1, 27)));
	leftHand->rotation.setValue(tmp_rot);

	//right hand
	rightHand->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 28), a.r(1, 29), a.r(1, 30))));

	//left foot
	leftFoot->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 31), a.r(1, 32), a.r(1, 33))));

	//right foot
	rightFoot->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 34), a.r(1, 35), a.r(1, 36))));

	//torso
	torso->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 37), a.r(1, 38), a.r(1, 39))));

	//pelvis
	pelvis->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 40), a.r(1, 41), a.r(1, 42))));

	//head
	head->rotation.setValue(Euler2Mat(SbVec3f( a.r(1, 43), a.r(1, 44), a.r(1, 45))));  
}


void
QuadrapedMap::read(M<double>& a)
{
	float x,y,z;
	SbRotation tmp_rot;

	// left arm
	leftArmCps->point.getValues(0)->getValue(x,y,z); 	
	a.r(1,1) = x; a.r(1,2) = y; a.r(1,3) = z;
	leftArmCps->point.getValues(1)->getValue(x,y,z);
	a.r(1,4) = x; a.r(1,5) = y; a.r(1,6) = z;

	// right arm
	rightArmCps->point.getValues(0)->getValue(x,y,z);
	a.r(1,7) = x; a.r(1,8) = y; a.r(1,9) = z;
	rightArmCps->point.getValues(1)->getValue(x,y,z);
	a.r(1,10) = x; a.r(1,11) = y; a.r(1,12) = z;

	// left leg
	leftLegCps->point.getValues(0)->getValue(x,y,z);
	a.r(1,13) = x; a.r(1,14) = y; a.r(1,15) = z;
	leftLegCps->point.getValues(1)->getValue(x,y,z);
	a.r(1,16) = x; a.r(1,17) = y; a.r(1,18) = z;

	// right leg
	rightLegCps->point.getValues(0)->getValue(x,y,z);
	a.r(1,19) = x; a.r(1,20) = y; a.r(1,21) = z;
	rightLegCps->point.getValues(1)->getValue(x,y,z);
	a.r(1,22) = x; a.r(1,23) = y; a.r(1,24) = z;

	M<double> tmp_vec;
       
	// left hand
	tmp_rot = leftHand->rotation.getValue();
	Mat2Euler(tmp_rot, tmp_vec);
    a(1, colon(25, 1, 27)) = tmp_vec(1, colon(1, 1, 3));
        
		
	//rightHand
	tmp_rot = rightHand->rotation.getValue();
    Mat2Euler(tmp_rot, tmp_vec);
    a(1, colon(28, 1, 30)) = tmp_vec(1, colon(1, 1, 3));
        
    
	//leftFoot
	Mat2Euler(leftFoot->rotation.getValue(), tmp_vec);
	a(1, colon(31, 1, 33)) = tmp_vec(1, colon(1, 1, 3));
	
	//rightFoot
	Mat2Euler(rightFoot->rotation.getValue(), tmp_vec);
	a(1, colon(34, 1, 36)) = tmp_vec(1, colon(1, 1, 3));
        
	//torso
	Mat2Euler(torso->rotation.getValue(), tmp_vec);
	a(1, colon(37, 1, 39)) = tmp_vec(1, colon(1, 1, 3));

	//pelvis
	Mat2Euler(pelvis->rotation.getValue(), tmp_vec);
	a(1, colon(40, 1, 42)) = tmp_vec(1, colon(1, 1, 3));

	//head
	Mat2Euler(head->rotation.getValue(), tmp_vec);
	a(1, colon(43, 1, 45)) = tmp_vec(1, colon(1, 1, 3));
      
}

void
QuadrapedMap::Mat2Euler(SbRotation rot, M<double>& eulVec)
	{
	SbMatrix rotMat;
	rot.getValue(rotMat);
	
	//might need to change all indexes back by one depending on how indexed	
	float tmp_x, tmp_y, tmp_z;
  
	//easy part... get values constrained to -PI/2 <= angle <= PI/2
	tmp_x = atan((double) (rotMat[1][2])/(rotMat[2][2])); 
	tmp_y = asin(-rotMat[0][2]);
	tmp_z = atan((double) (rotMat[0][1])/(rotMat[0][0]));
  
	//now need to place in correct quadrants
	eulVec(1, 1) = tmp_x;
	eulVec(1, 2) = tmp_y;
	eulVec(1, 3) = tmp_z;
	}

SbMatrix
QuadrapedMap::Euler2Mat(SbVec3f eulVec)
	{
	//matrix 4 by 4 auutomaticaly
	float x, y, z;
	eulVec.getValue(x, y, z);
  
	float cx = cos(x);
	float sx = sin(x);
	float cy = cos(y);
	float sy = sin(y);
	float cz = cos(z);
	float sz = sin(z);

	SbMatrix rotMat;

	rotMat[0][0] = cy*cz;
	rotMat[0][1] = cy*sz;
	rotMat[0][2] = -sy;
	rotMat[0][3] = 0;
	rotMat[1][0] = sx*sy*cz - cx*sz;
	rotMat[1][1] = sx*sy*sz + cx*cz;
	rotMat[1][2] = sx*cy;
	rotMat[1][3] = 0;
	rotMat[2][0] = cx*sy*cz + sx*sz;
	rotMat[2][1] = cx*sy*sz - sx*cz;
	rotMat[2][2] = cx*cy;
	rotMat[2][3] = 0;
	rotMat[3][0] = 0;
	rotMat[3][1] = 0;
	rotMat[3][2] = 0;
	rotMat[3][3] = 1;
    
	/*rotMat[0][0] = rotMat[1][1] = rotMat[2][2] = rotMat[3][3] = 1;
	rotMat[0][1] = rotMat[0][2] = rotMat[0][3] = rotMat[1][0] = rotMat[1][2] = rotMat[1][3] = rotMat[2][0] = rotMat[2][1] = rotMat[2][3] = rotMat[3][0] = rotMat[3][1] = rotMat[3][2] = 0;
	*/

	return rotMat;
	}




